/**
 * @NApiVersion 2.x
 * @NScriptType Restlet
 * @NModuleScope Public
 */

var SCRIPT_VERSION = 3;

/**
 * Upload this file to your Netsuite file cabinet and deploy the script. Please check out our
 * documentation for more information:
 * https://docs.causal.app/data/integrations/accounting-erp/netsuite#saved-searches
 */

define(['N/search'],
    /**
     * @param {search} searchModule - Saved Search Module
     */
    function (searchModule) {

        var searchColumns = [];

        /**
         * Function called upon sending a GET request to the RESTlet.
         *
         * @param {Object} requestParams - Parameters from HTTP request URL; parameters will be passed into function as an Object (for all supported content types)
         * @returns {string | Object} HTTP response body; return string when request Content-Type is 'text/plain'; return Object when request Content-Type is 'application/json'
         * @since 2015.1
         */
        function doGet(requestParams) {
            try {
                var startTime = new Date().getTime();
                
                log.debug('requestParams', requestParams);

                // searchType is optional, e.g. it doesn't have to be provided for transaction searches
                var searchType = requestParams.searchType;
                var searchId = requestParams.searchId;

                var pageIndex = requestParams.pageIndex;
                // not provided on first call
                if (!pageIndex) {
                    pageIndex = 0;
                }

                // maximum script run time in ms. If maximum page size is reached before this time, we end the script
                // by default we stop processing after 4 minutes (max total RESTlet execution time is 5 minutes)
                var customTime = requestParams.customTime;
                if (!customTime) {
                    customTime = 4 * 60 * 1000;
                }
                var endTime = startTime + customTime;

                // maximum page size. If time is up before page size is reached, we end the script
                var pageSize = requestParams.pageSize;
                if (!pageSize) {
                    pageSize = 10000;
                }


                var searchObject = searchModule.create({
                    type: searchModule.Type.SAVED_SEARCH,
                    filters:
                        [
                            ["internalid", "is", searchId]
                        ],
                    columns:
                        []
                });
                var searchResultCount = searchObject.runPaged().count;
                log.debug("searchObject result count", searchResultCount);


                if (searchResultCount == 0) {
                    return { error: 'Search with the id ' + searchId + ' cannot be found.' };
                }

                
                // if search type has been provided, use this to load the search
                // otherwise, load normally
                var loadedSearch;
                if (searchType) {
                    var loadedSearch = searchModule.load({
                        id: searchId,
                        type: searchType
                    });
                } else {
                    var loadedSearch = searchModule.load({
                        id: searchId
                    });
                }

                var resultsArray = [];
                searchColumns = loadedSearch.columns;
                log.debug('searchColumns', searchColumns);

                var results = loadedSearch.run();
                var sliceIndex = parseInt(pageIndex);
                do {
                    var resultSlice = results.getRange({ start: sliceIndex, end: sliceIndex + 1000 });
                    resultSlice.forEach(function (slice) {
                        resultsArray.push(resToObj(slice));
                        sliceIndex++;
                    }
                    );
                    var isFinished = resultSlice.length < 1000;
                    var reachedPageSize = resultsArray.length >= pageSize;
                    var timeIsUp = new Date().getTime() > endTime;
                } while (!isFinished && !timeIsUp && !reachedPageSize);

                log.debug('resultsArray[0:10]', resultsArray.slice(0, 10));

                return { data: resultsArray, isFinished: isFinished, pageIndex: sliceIndex, scriptVersion: SCRIPT_VERSION };

            } catch (e) {
                log.error('Error retrieving search: e', e);
                log.error('Error retrieving search: e.message', e.message);
                return { error: e.message };
            }
        }

        function resToObj(result) {
            try {
                var obj = {};

                for (var i = 0; i < searchColumns.length; i++) {
                    var curColumnName = searchColumns[i].name;
                    var curColumnJoin = searchColumns[i].join;
                    var curColumnSummary = searchColumns[i].summary;
                    var label = searchColumns[i].label;

                    //if there is a join - get results differently
                    if (curColumnSummary) {
                        var summaryType;
                        switch (curColumnSummary) {
                            case "GROUP":
                                summaryType = searchModule.Summary.GROUP;
                                break;
                            case "COUNT":
                                summaryType = searchModule.Summary.COUNT;
                                break;
                            case "SUM":
                                summaryType = searchModule.Summary.SUM;
                                break;
                            case "MIN":
                                summaryType = searchModule.Summary.MIN;
                                break;
                            case "MAX":
                                summaryType = searchModule.Summary.MAX;
                                break;
                            case "AVG":
                                summaryType = searchModule.Summary.AVG;
                                break;
                        }

                        if (curColumnJoin) {
                            obj[label] = {
                                value: result.getValue({ name: curColumnName, summary: summaryType, join: curColumnJoin }),
                                text: result.getText({ name: curColumnName, summary: summaryType, join: curColumnJoin })
                            };
                        } else {
                            obj[label] = {
                                value: result.getValue({ name: curColumnName, summary: summaryType }),
                                text: result.getText({ name: curColumnName, summary: summaryType })
                            };
                        }
                    }
                    else if (curColumnJoin) {
                        obj[label] = {
                            value: result.getValue({ name: curColumnName, join: curColumnJoin }),
                            text: result.getText({ name: curColumnName, join: curColumnJoin })
                        };
                    } else {
                        obj[label] = {
                            value: result.getValue(curColumnName),
                            text: result.getText(curColumnName),
                        };
                    }
                }
                return obj
            } catch (e) {
                log.error({ title: "Caught Exception resultToObject", "details": e.message || "No Message" });
            }
        }


        /**
         * Function called upon sending a PUT request to the RESTlet.
         *
         * @param {string | Object} requestBody - The HTTP request body; request body will be passed into function as a string when request Content-Type is 'text/plain'
         * or parsed into an Object when request Content-Type is 'application/json' (in which case the body must be a valid JSON)
         * @returns {string | Object} HTTP response body; return string when request Content-Type is 'text/plain'; return Object when request Content-Type is 'application/json'
         * @since 2015.2
         */
        function doPut(requestBody) {
            // not used
        }


        /**
         * Function called upon sending a POST request to the RESTlet.
         *
         * @param {string | Object} requestBody - The HTTP request body; request body will be passed into function as a string when request Content-Type is 'text/plain'
         * or parsed into an Object when request Content-Type is 'application/json' (in which case the body must be a valid JSON)
         * @returns {string | Object} HTTP response body; return string when request Content-Type is 'text/plain'; return Object when request Content-Type is 'application/json'
         * @since 2015.2
         */
        function doPost(requestBody) {
            // not used
        }


        /**
         * Function called upon sending a DELETE request to the RESTlet.
         *
         * @param {Object} requestParams - Parameters from HTTP request URL; parameters will be passed into function as an Object (for all supported content types)
         * @returns {string | Object} HTTP response body; return string when request Content-Type is 'text/plain'; return Object when request Content-Type is 'application/json'
         * @since 2015.2
         */
        function doDelete(requestParams) {
            // not used
        }


        return {
            'get': doGet,
            'put': doPut,
            'post': doPost,
            'delete': doDelete
        };
    });